@@ -1,9 +1,9 @@
-/* Copyright (c) 2002-2012 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2002-2013 Dovecot authors, see the included COPYING file */
 
 #include "auth-common.h"
 #include "array.h"
 #include "ioloop.h"
-#include "network.h"
+#include "net.h"
 #include "lib-signals.h"
 #include "restrict-access.h"
 #include "child-wait.h"
@@ -22,6 +22,7 @@
 #include "mech.h"
 #include "auth.h"
 #include "auth-penalty.h"
+#include "auth-token.h"
 #include "auth-request-handler.h"
 #include "auth-worker-server.h"
 #include "auth-worker-client.h"
@@ -40,7 +41,9 @@ enum auth_socket_type {
 	AUTH_SOCKET_LOGIN_CLIENT,
 	AUTH_SOCKET_MASTER,
 	AUTH_SOCKET_USERDB,
-	AUTH_SOCKET_POSTFIX
+	AUTH_SOCKET_POSTFIX,
+	AUTH_SOCKET_TOKEN,
+	AUTH_SOCKET_TOKEN_LOGIN
 };
 
 struct auth_socket_listener {
@@ -56,7 +59,7 @@ struct auth_penalty *auth_penalty;
 static pool_t auth_set_pool;
 static struct module *modules = NULL;
 static struct mechanisms_register *mech_reg;
-static ARRAY_DEFINE(listeners, struct auth_socket_listener);
+static ARRAY(struct auth_socket_listener) listeners;
 
 void auth_refresh_proctitle(void)
 {
@@ -118,6 +121,10 @@ auth_socket_type_get(const char *path)
 		return AUTH_SOCKET_USERDB;
 	else if (strcmp(suffix, "postmap") == 0)
 		return AUTH_SOCKET_POSTFIX;
+	else if (strcmp(suffix, "token") == 0)
+		return AUTH_SOCKET_TOKEN;
+	else if (strcmp(suffix, "tokenlogin") == 0)
+		return AUTH_SOCKET_TOKEN_LOGIN;
 	else
 		return AUTH_SOCKET_CLIENT;
 }
@@ -181,7 +188,7 @@ static void main_preinit(void)
 	services = read_global_settings();
 
 	memset(&mod_set, 0, sizeof(mod_set));
-	mod_set.version = master_service_get_version_string(master_service);
+	mod_set.abi_version = DOVECOT_ABI_VERSION;
 	mod_set.require_init_funcs = TRUE;
 	mod_set.debug = global_auth_settings->debug;
 	mod_set.filter_callback = auth_module_filter;
@@ -198,6 +205,8 @@ static void main_preinit(void)
 		      mech_reg, services);
 
 	listeners_init();
+	if (!worker)
+		auth_token_init();
 
 	/* Password lookups etc. may require roots, allow it. */
 	restrict_access_by_env(NULL, FALSE);
@@ -209,7 +218,7 @@ void auth_module_load(const char *names)
 	struct module_dir_load_settings mod_set;
 
 	memset(&mod_set, 0, sizeof(mod_set));
-	mod_set.version = master_service_get_version_string(master_service);
+	mod_set.abi_version = DOVECOT_ABI_VERSION;
 	mod_set.require_init_funcs = TRUE;
 	mod_set.debug = global_auth_settings->debug;
 	mod_set.ignore_missing = TRUE;
@@ -229,7 +238,7 @@ static void main_init(void)
 
 	/* set proctitles before init()s, since they may set them to error */
 	auth_refresh_proctitle();
-	auth_worker_refresh_proctitle(NULL);
+	auth_worker_refresh_proctitle("");
 
 	child_wait_init();
 	auth_worker_server_init();
@@ -264,6 +273,8 @@ static void main_deinit(void)
 	auths_free();
 	dict_drivers_unregister_builtin();
 
+	auth_token_deinit();
+
 	auth_client_connections_destroy_all();
 	auth_master_connections_destroy_all();
 	auth_postfix_connections_destroy_all();
@@ -300,7 +311,7 @@ static void worker_connected(struct master_service_connection *conn)
 	}
 
 	master_service_client_connection_accept(conn);
-	(void)auth_worker_client_create(auth_find_service(NULL), conn->fd);
+	(void)auth_worker_client_create(auth_default_service(), conn->fd);
 }
 
 static void client_connected(struct master_service_connection *conn)
@@ -315,7 +326,7 @@ static void client_connected(struct master_service_connection *conn)
 		l->type = auth_socket_type_get(conn->name);
 		l->path = i_strdup(conn->name);
 	}
-	auth = auth_find_service(NULL);
+	auth = auth_default_service();
 	switch (l->type) {
 	case AUTH_SOCKET_MASTER:
 		(void)auth_master_connection_create(auth, conn->fd,
@@ -329,10 +340,16 @@ static void client_connected(struct master_service_connection *conn)
 		(void)auth_postfix_connection_create(auth, conn->fd);
 		break;
 	case AUTH_SOCKET_LOGIN_CLIENT:
-		(void)auth_client_connection_create(auth, conn->fd, TRUE);
+		auth_client_connection_create(auth, conn->fd, TRUE, FALSE);
 		break;
 	case AUTH_SOCKET_CLIENT:
-		(void)auth_client_connection_create(auth, conn->fd, FALSE);
+		auth_client_connection_create(auth, conn->fd, FALSE, FALSE);
+		break;
+	case AUTH_SOCKET_TOKEN_LOGIN:
+		auth_client_connection_create(auth, conn->fd, TRUE, TRUE);
+		break;
+	case AUTH_SOCKET_TOKEN:
+		auth_client_connection_create(auth, conn->fd, FALSE, TRUE);
 		break;
 	default:
 		i_unreached();
