@@ -5,19 +5,17 @@
 #include "mail-index-private.h"
 #include "mail-cache.h"
 
-#define MAIL_CACHE_VERSION 1
+#define MAIL_CACHE_MAJOR_VERSION 1
+#define MAIL_CACHE_MINOR_VERSION 1
 
 /* Drop fields that haven't been accessed for n seconds */
 #define MAIL_CACHE_FIELD_DROP_SECS (3600*24*30)
 
 /* Never compress the file if it's smaller than this */
-#define MAIL_CACHE_COMPRESS_MIN_SIZE (1024*50)
+#define MAIL_CACHE_COMPRESS_MIN_SIZE (1024*32)
 
-/* Don't bother remembering holes smaller than this */
-#define MAIL_CACHE_MIN_HOLE_SIZE 1024
-
-/* Compress the file when deleted space reaches n% of total size */
-#define MAIL_CACHE_COMPRESS_PERCENTAGE 20
+/* Compress the file when n% of records are deleted */
+#define MAIL_CACHE_COMPRESS_DELETE_PERCENTAGE 20
 
 /* Compress the file when n% of rows contain continued rows.
    200% means that there's 2 continued rows per record. */
@@ -27,40 +25,33 @@
    the latest cache header. */
 #define MAIL_CACHE_HEADER_FIELD_CONTINUE_COUNT 4
 
-/* Initial size for the file */
-#define MAIL_CACHE_INITIAL_SIZE (sizeof(struct mail_cache_header) + 10240)
-
-/* When more space is needed, grow the file n% larger than the previous size */
-#define MAIL_CACHE_GROW_PERCENTAGE 10
-
-/* When allocating space for transactions, don't use blocks larger than this. */
-#define MAIL_CACHE_MAX_RESERVED_BLOCK_SIZE (1024*512)
+/* If cache record becomes larger than this, don't add it. */
+#define MAIL_CACHE_RECORD_MAX_SIZE (64*1024)
 
 #define MAIL_CACHE_LOCK_TIMEOUT 10
 #define MAIL_CACHE_LOCK_CHANGE_TIMEOUT 300
 
-#define CACHE_RECORD(cache, offset) \
-	((const struct mail_cache_record *) \
-	 ((const char *) (cache)->data + offset))
-
 #define MAIL_CACHE_IS_UNUSABLE(cache) \
 	((cache)->hdr == NULL)
 
 struct mail_cache_header {
 	/* version is increased only when you can't have backwards
 	   compatibility. */
-	uint8_t version;
+	uint8_t major_version;
 	uint8_t compat_sizeof_uoff_t;
-	uint8_t unused[2];
+	uint8_t minor_version;
+	uint8_t unused;
 
 	uint32_t indexid;
 	uint32_t file_seq;
 
 	uint32_t continued_record_count;
 
-	uint32_t hole_offset;
-	uint32_t used_file_size;
-	uint32_t deleted_space;
+	/* NOTE: old versions used this for hole offset, so we can't fully
+	   rely on it */
+	uint32_t record_count;
+	uint32_t backwards_compat_used_file_size;
+	uint32_t deleted_record_count;
 
 	uint32_t field_header_offset;
 };
@@ -102,17 +93,6 @@ struct mail_cache_record {
 	/* array of { uint32_t field; [ uint32_t size; ] { .. } } */
 };
 
-struct mail_cache_hole_header {
-	uint32_t next_offset; /* 0 if no holes left */
-	uint32_t size; /* including this header */
-
-	/* make sure we notice if we're treating hole as mail_cache_record.
-	   magic is a large number so if it's treated as size field, it'll
-	   point outside the file */
-#define MAIL_CACHE_HOLE_HEADER_MAGIC 0xffeedeff
-	uint32_t magic;
-};
-
 struct mail_cache_field_private {
 	struct mail_cache_field field;
 
@@ -134,10 +114,14 @@ struct mail_cache {
 	ino_t st_ino;
 	dev_t st_dev;
 
-	void *mmap_base;
-	const void *data;
 	size_t mmap_length;
+	/* a) mmaping the whole file */
+	void *mmap_base;
+	/* b) using file cache */
 	struct file_cache *file_cache;
+	/* c) using small read() calls with MAIL_INDEX_OPEN_FLAG_SAVEONLY */
+	uoff_t read_offset;
+	buffer_t *read_buf;
 	/* mail_cache_map() increases this always. */
 	unsigned int remap_counter;
 
@@ -158,7 +142,7 @@ struct mail_cache {
 	struct mail_cache_field_private *fields;
 	uint32_t *field_file_map;
 	unsigned int fields_count;
-	struct hash_table *field_name_hash; /* name -> idx */
+	HASH_TABLE(char *, void *) field_name_hash; /* name -> idx */
 	uint32_t last_field_header_offset;
 
 	/* 0 is no need for compression, otherwise the file sequence number
@@ -174,6 +158,7 @@ struct mail_cache {
 	unsigned int hdr_modified:1;
 	unsigned int field_header_write_pending:1;
 	unsigned int compressing:1;
+	unsigned int map_with_read:1;
 };
 
 struct mail_cache_loop_track {
@@ -203,8 +188,9 @@ struct mail_cache_view {
 
 struct mail_cache_iterate_field {
 	unsigned int field_idx;
-	const void *data;
 	unsigned int size;
+	const void *data;
+	uoff_t offset;
 };
 
 struct mail_cache_lookup_iterate_ctx {
@@ -230,6 +216,8 @@ int mail_cache_unlock(struct mail_cache *cache);
 
 int mail_cache_write(struct mail_cache *cache, const void *data, size_t size,
 		     uoff_t offset);
+int mail_cache_append(struct mail_cache *cache, const void *data, size_t size,
+		      uint32_t *offset);
 
 int mail_cache_header_fields_read(struct mail_cache *cache);
 int mail_cache_header_fields_update(struct mail_cache *cache);
@@ -256,15 +244,12 @@ void mail_cache_lookup_iter_init(struct mail_cache_view *view, uint32_t seq,
 int mail_cache_lookup_iter_next(struct mail_cache_lookup_iterate_ctx *ctx,
 				struct mail_cache_iterate_field *field_r);
 
-int mail_cache_map(struct mail_cache *cache, size_t offset, size_t size);
+int mail_cache_map(struct mail_cache *cache, size_t offset, size_t size,
+		   const void **data_r);
 void mail_cache_file_close(struct mail_cache *cache);
 int mail_cache_reopen(struct mail_cache *cache);
 
-/* Update new_offset's prev_offset field to old_offset. */
-int mail_cache_link(struct mail_cache *cache, uint32_t old_offset,
-		    uint32_t new_offset);
-/* Mark record in given offset to be deleted. */
-int mail_cache_delete(struct mail_cache *cache, uint32_t offset);
+void mail_cache_delete(struct mail_cache *cache);
 
 /* Notify the decision handling code that field was looked up for seq.
    This should be called even for fields that aren't currently in cache file */
@@ -276,10 +261,6 @@ void mail_cache_decision_add(struct mail_cache_view *view, uint32_t seq,
 int mail_cache_expunge_handler(struct mail_index_sync_map_ctx *sync_ctx,
 			       uint32_t seq, const void *data,
 			       void **sync_context, void *context);
-int mail_cache_sync_handler(struct mail_index_sync_map_ctx *sync_ctx,
-			    uint32_t seq, void *old_data, const void *new_data,
-			    void **context);
-void mail_cache_sync_lost_handler(struct mail_index *index);
 
 void mail_cache_set_syscall_error(struct mail_cache *cache,
 				  const char *function);
