@@ -1,4 +1,4 @@
-/* Copyright (c) 2004-2012 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2004-2013 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "array.h"
@@ -310,7 +310,7 @@ sync_ext_resize(const struct mail_transaction_ext_intro *u,
 		i_assert((map->hdr_copy_buf->used % sizeof(uint64_t)) == 0);
 		map->hdr_base = map->hdr_copy_buf->data;
 		map->hdr.header_size = map->hdr_copy_buf->used;
-		map->write_base_header = map->write_ext_header = TRUE;
+		map->header_changed = TRUE;
 
 		ext_hdr = get_ext_header(map, ext);
 		ext_hdr->reset_id = ext->reset_id;
@@ -570,8 +570,7 @@ static void mail_index_sync_ext_clear(struct mail_index_view *view,
 		memset(PTR_OFFSET(rec, ext->record_offset), 0,
 		       ext->record_size);
 	}
-	map->rec_map->write_seq_first = 1;
-	map->rec_map->write_seq_last = view->map->rec_map->records_count;
+	map->rec_map->records_changed = TRUE;
 }
 
 int mail_index_sync_ext_reset(struct mail_index_sync_map_ctx *ctx,
@@ -635,7 +634,7 @@ int mail_index_sync_ext_hdr_update(struct mail_index_sync_map_ctx *ctx,
 	if (ext->index_idx == ctx->view->index->modseq_ext_id)
 		mail_index_modseq_hdr_update(ctx->modseq_ctx);
 
-	map->write_ext_header = TRUE;
+	map->header_changed = TRUE;
 	return 1;
 }
 
@@ -684,7 +683,7 @@ mail_index_sync_ext_rec_update(struct mail_index_sync_map_ctx *ctx,
 			return ret;
 	}
 
-	mail_index_sync_write_seq_update(ctx, seq, seq);
+	view->map->rec_map->records_changed = TRUE;
 
 	/* @UNSAFE */
 	memcpy(old_data, u + 1, ext->record_size);
@@ -786,6 +785,6 @@ mail_index_sync_ext_atomic_inc(struct mail_index_sync_map_ctx *ctx,
 		return -1;
 	}
 
-	mail_index_sync_write_seq_update(ctx, seq, seq);
+	view->map->rec_map->records_changed = TRUE;
 	return 1;
 }
