@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2012 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2002-2013 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "ioloop.h"
@@ -38,6 +38,7 @@
 #define SEARCH_RECALC_MIN_USECS 50000
 
 struct search_header_context {
+        struct index_search_context *index_ctx;
         struct index_mail *imail;
 	struct mail_search_arg *args;
 
@@ -58,8 +59,8 @@ static void search_parse_msgset_args(unsigned int messages_count,
 				     struct mail_search_arg *args,
 				     uint32_t *seq1_r, uint32_t *seq2_r);
 
-static void search_none(struct mail_search_arg *arg ATTR_UNUSED,
-			struct search_body_context *ctx ATTR_UNUSED)
+static void ATTR_NULL(2)
+search_none(struct mail_search_arg *arg ATTR_UNUSED, void *ctx ATTR_UNUSED)
 {
 }
 
@@ -147,12 +148,24 @@ static int search_arg_match_keywords(struct index_search_context *ctx,
 	return 1;
 }
 
+static bool
+index_search_get_pvt(struct index_search_context *ctx, uint32_t uid)
+{
+	index_transaction_init_pvt(ctx->mail_ctx.transaction);
+
+	if (ctx->pvt_uid == uid)
+		return ctx->pvt_seq != 0;
+	ctx->pvt_uid = uid;
+	return mail_index_lookup_seq(ctx->mail_ctx.transaction->view_pvt,
+				     uid, &ctx->pvt_seq);
+}
+
 /* Returns >0 = matched, 0 = not matched, -1 = unknown */
 static int search_arg_match_index(struct index_search_context *ctx,
 				  struct mail_search_arg *arg,
 				  const struct mail_index_record *rec)
 {
-	enum mail_flags flags;
+	enum mail_flags flags, pvt_flags_mask;
 	uint64_t modseq;
 	int ret;
 
@@ -167,6 +180,18 @@ static int search_arg_match_index(struct index_search_context *ctx,
 		if ((arg->value.flags & MAIL_RECENT) != 0 &&
 		    index_mailbox_is_recent(ctx->box, rec->uid))
 			flags |= MAIL_RECENT;
+		if (ctx->box->view_pvt == NULL) {
+			/* no private view (set by view syncing) ->
+			   no private flags */
+		} else {
+			pvt_flags_mask = mailbox_get_private_flags_mask(ctx->box);
+			flags &= ~pvt_flags_mask;
+			if (index_search_get_pvt(ctx, rec->uid)) {
+				rec = mail_index_lookup(ctx->mail_ctx.transaction->view_pvt,
+							ctx->pvt_seq);
+				flags |= rec->flags & pvt_flags_mask;
+			}
+		}
 		return (flags & arg->value.flags) == arg->value.flags;
 	case SEARCH_KEYWORDS:
 		T_BEGIN {
@@ -372,16 +397,16 @@ static int search_sent(enum mail_search_arg_type type, time_t search_time,
 }
 
 static struct message_search_context *
-msg_search_arg_context(struct mail_search_arg *arg)
+msg_search_arg_context(struct index_search_context *ctx,
+		       struct mail_search_arg *arg)
 {
-	enum message_search_flags flags = MESSAGE_SEARCH_FLAG_DTCASE;
+	enum message_search_flags flags = 0;
 
 	if (arg->context == NULL) T_BEGIN {
 		string_t *dtc = t_str_new(128);
 
-		if (uni_utf8_to_decomposed_titlecase(arg->value.str,
-						     strlen(arg->value.str),
-						     dtc) < 0)
+		if (ctx->mail_ctx.normalizer(arg->value.str,
+					     strlen(arg->value.str), dtc) < 0)
 			i_panic("search key not utf8: %s", arg->value.str);
 
 		if (arg->type == SEARCH_BODY)
@@ -389,8 +414,12 @@ msg_search_arg_context(struct mail_search_arg *arg)
 		/* we don't get here if arg is "", but dtc can be "" if it
 		   only contains characters that we need to ignore. handle
 		   those searches by returning them as non-matched. */
-		if (str_len(dtc) > 0)
-			arg->context = message_search_init(str_c(dtc), flags);
+		if (str_len(dtc) > 0) {
+			arg->context =
+				message_search_init(str_c(dtc),
+						    ctx->mail_ctx.normalizer,
+						    flags);
+		}
 	} T_END;
 	return arg->context;
 }
@@ -475,7 +504,7 @@ static void search_header_arg(struct mail_search_arg *arg,
 	hdr.middle_len = 0;
 	block.hdr = &hdr;
 
-	msg_search_ctx = msg_search_arg_context(arg);
+	msg_search_ctx = msg_search_arg_context(ctx->index_ctx, arg);
 	if (msg_search_ctx == NULL)
 		return;
 
@@ -492,7 +521,7 @@ static void search_header_arg(struct mail_search_arg *arg,
 			addr = message_address_parse(pool_datastack_create(),
 						     ctx->hdr->full_value,
 						     ctx->hdr->full_value_len,
-						     (unsigned int)-1, TRUE);
+						     UINT_MAX, TRUE);
 			str = t_str_new(ctx->hdr->value_len);
 			message_address_write(str, addr);
 			hdr.value = hdr.full_value = str_data(str);
@@ -517,7 +546,7 @@ static void search_header_arg(struct mail_search_arg *arg,
 }
 
 static void search_header_unmatch(struct mail_search_arg *arg,
-				  void *context ATTR_UNUSED)
+				  struct search_header_context *ctx ATTR_UNUSED)
 {
 	switch (arg->type) {
 	case SEARCH_BEFORE:
@@ -547,7 +576,8 @@ static void search_header(struct message_header_line *hdr,
 {
 	if (hdr == NULL) {
 		/* end of headers, mark all unknown SEARCH_HEADERs unmatched */
-		mail_search_args_foreach(ctx->args, search_header_unmatch, ctx);
+		(void)mail_search_args_foreach(ctx->args, search_header_unmatch,
+					       ctx);
 		return;
 	}
 
@@ -561,7 +591,7 @@ static void search_header(struct message_header_line *hdr,
 		ctx->hdr = hdr;
 
 		ctx->custom_header = FALSE;
-		mail_search_args_foreach(ctx->args, search_header_arg, ctx);
+		(void)mail_search_args_foreach(ctx->args, search_header_arg, ctx);
 	}
 }
 
@@ -579,7 +609,7 @@ static void search_body(struct mail_search_arg *arg,
 		return;
 	}
 
-	msg_search_ctx = msg_search_arg_context(arg);
+	msg_search_ctx = msg_search_arg_context(ctx->index_ctx, arg);
 	if (msg_search_ctx == NULL) {
 		ARG_SET_RESULT(arg, 0);
 		return;
@@ -596,6 +626,10 @@ static void search_body(struct mail_search_arg *arg,
 		ret = message_search_msg(msg_search_ctx, ctx->input, NULL);
 		i_assert(ret >= 0 || ctx->input->stream_errno != 0);
 	}
+	if (ctx->input->stream_errno != 0) {
+		mail_storage_set_critical(ctx->index_ctx->box->storage,
+			"read(%s) failed: %m", i_stream_get_name(ctx->input));
+	}
 
 	ARG_SET_RESULT(arg, ret);
 }
@@ -620,6 +654,7 @@ static int search_arg_match_text(struct mail_search_arg *args,
 		return -1;
 
 	memset(&hdr_ctx, 0, sizeof(hdr_ctx));
+	hdr_ctx.index_ctx = ctx;
 	/* hdr_ctx.imail is different from imail for mails in
 	   virtual mailboxes */
 	hdr_ctx.imail = (struct index_mail *)mail_get_real_mail(ctx->cur_mail);
@@ -655,6 +690,11 @@ static int search_arg_match_text(struct mail_search_arg *args,
 			}
 			message_parse_header(input, NULL, hdr_parser_flags,
 					     search_header, &hdr_ctx);
+			if (input->stream_errno != 0) {
+				mail_storage_set_critical(ctx->box->storage,
+					"read(%s) failed: %m", i_stream_get_name(input));
+				failed = TRUE;
+			}
 		}
 	}
 	if (headers_ctx != NULL)
@@ -672,7 +712,7 @@ static int search_arg_match_text(struct mail_search_arg *args,
 
 	if (have_headers) {
 		/* see if the header search succeeded in finishing the search */
-		ret = mail_search_args_foreach(args, search_none, NULL);
+		ret = mail_search_args_foreach(args, search_none, (void *)NULL);
 		if (ret >= 0 || !have_body)
 			return ret;
 	}
@@ -716,7 +756,7 @@ search_msgset_fix_limits(unsigned int messages_count,
 		if (range[count-1].seq2 == (uint32_t)-1) {
 			/* "*" used, make sure the last message is in the range
 			   (e.g. with count+1:* we still want to include it) */
-			seq_range_array_add(seqset, 0, messages_count);
+			seq_range_array_add(seqset, messages_count);
 		}
 		/* remove all nonexistent messages */
 		seq_range_array_remove_range(seqset, messages_count + 1,
@@ -858,8 +898,8 @@ static void search_limit_lowwater(struct index_search_context *ctx,
 	if (uid_lowwater == 0)
 		return;
 
-	mail_index_lookup_seq_range(ctx->view, uid_lowwater, (uint32_t)-1,
-				    &seq1, &seq2);
+	(void)mail_index_lookup_seq_range(ctx->view, uid_lowwater, (uint32_t)-1,
+					  &seq1, &seq2);
 	if (*first_seq < seq1)
 		*first_seq = seq1;
 }
@@ -869,8 +909,14 @@ static bool search_limit_by_flags(struct index_search_context *ctx,
 				  uint32_t *seq1, uint32_t *seq2)
 {
 	const struct mail_index_header *hdr;
+	enum mail_flags pvt_flags_mask;
 
 	hdr = mail_index_get_header(ctx->view);
+	/* we can't trust that private view's header is fully up to date,
+	   so do this optimization only for non-private flags */
+	pvt_flags_mask = ctx->box->view_pvt == NULL ? 0 :
+		mailbox_get_private_flags_mask(ctx->box);
+
 	for (; args != NULL; args = args->next) {
 		if (args->type != SEARCH_FLAGS) {
 			if (args->type == SEARCH_ALL) {
@@ -879,7 +925,8 @@ static bool search_limit_by_flags(struct index_search_context *ctx,
 			}
 			continue;
 		}
-		if ((args->value.flags & MAIL_SEEN) != 0) {
+		if ((args->value.flags & MAIL_SEEN) != 0 &&
+		    (pvt_flags_mask & MAIL_SEEN) == 0) {
 			/* SEEN with 0 seen? */
 			if (!args->match_not && hdr->seen_messages_count == 0)
 				return FALSE;
@@ -897,14 +944,14 @@ static bool search_limit_by_flags(struct index_search_context *ctx,
                                 	hdr->first_unseen_uid_lowwater, seq1);
 			}
 		}
-		if ((args->value.flags & MAIL_DELETED) != 0) {
+		if ((args->value.flags & MAIL_DELETED) != 0 &&
+		    (pvt_flags_mask & MAIL_DELETED) == 0) {
 			/* DELETED with 0 deleted? */
 			if (!args->match_not &&
 			    hdr->deleted_messages_count == 0)
 				return FALSE;
 
-			if (hdr->deleted_messages_count ==
-			    hdr->messages_count) {
+			if (hdr->deleted_messages_count == hdr->messages_count) {
 				/* UNDELETED with all deleted? */
 				if (args->match_not)
 					return FALSE;
@@ -968,7 +1015,7 @@ static int search_build_subthread(struct mail_thread_iterate_context *iter,
 			if (search_build_subthread(child_iter, uids) < 0)
 				ret = -1;
 		}
-		seq_range_array_add(uids, 0, node->uid);
+		seq_range_array_add(uids, node->uid);
 	}
 	if (mail_thread_iterate_deinit(&iter) < 0)
 		ret = -1;
@@ -1009,7 +1056,7 @@ static int search_build_inthread_result(struct index_search_context *ctx,
 	iter = mail_thread_iterate_init(ctx->thread_ctx,
 					arg->value.thread_type, FALSE);
 	while ((node = mail_thread_iterate_next(iter, &child_iter)) != NULL) {
-		seq_range_array_add(&thread_uids, 0, node->uid);
+		seq_range_array_add(&thread_uids, node->uid);
 		if (child_iter != NULL) {
 			if (search_build_subthread(child_iter,
 						   &thread_uids) < 0)
@@ -1100,7 +1147,7 @@ wanted_sort_fields_get(struct mailbox *box,
 	}
 
 	if (array_count(&headers) > 0) {
-		(void)array_append_space(&headers);
+		array_append_zero(&headers);
 		*headers_ctx_r = mailbox_header_lookup_init(box,
 							array_idx(&headers, 0));
 	}
@@ -1118,6 +1165,7 @@ index_storage_search_init(struct mailbox_transaction_context *t,
 
 	ctx = i_new(struct index_search_context, 1);
 	ctx->mail_ctx.transaction = t;
+	ctx->mail_ctx.normalizer = t->box->storage->user->default_normalizer;
 	ctx->box = t->box;
 	ctx->view = t->view;
 	ctx->mail_ctx.args = args;
@@ -1125,7 +1173,7 @@ index_storage_search_init(struct mailbox_transaction_context *t,
 
 	ctx->max_mails = t->box->storage->set->mail_prefetch_count + 1;
 	if (ctx->max_mails == 0)
-		ctx->max_mails = -1U;
+		ctx->max_mails = UINT_MAX;
 	ctx->next_time_check_cost = SEARCH_INITIAL_MAX_COST;
 	if (gettimeofday(&ctx->last_nonblock_timeval, NULL) < 0)
 		i_fatal("gettimeofday() failed: %m");
@@ -1164,8 +1212,9 @@ index_storage_search_init(struct mailbox_transaction_context *t,
 	return &ctx->mail_ctx;
 }
 
-static void search_arg_deinit(struct mail_search_arg *arg,
-			      void *context ATTR_UNUSED)
+static void ATTR_NULL(2)
+search_arg_deinit(struct mail_search_arg *arg,
+		  struct index_search_context *ctx ATTR_UNUSED)
 {
 	struct message_search_context *search_ctx = arg->context;
 
@@ -1185,7 +1234,7 @@ int index_storage_search_deinit(struct mail_search_context *_ctx)
 
 	mail_search_args_reset(ctx->mail_ctx.args->args, FALSE);
 	(void)mail_search_args_foreach(ctx->mail_ctx.args->args,
-				       search_arg_deinit, NULL);
+				       search_arg_deinit, ctx);
 
 	if (ctx->mail_ctx.wanted_headers != NULL)
 		mailbox_header_lookup_unref(&ctx->mail_ctx.wanted_headers);
@@ -1449,7 +1498,7 @@ static int search_more_with_mail(struct index_search_context *ctx,
 			/* result isn't known yet, do a prefetch and
 			   finish later */
 			imail->data.search_results =
-				buffer_create_dynamic(imail->data_pool, 64);
+				buffer_create_dynamic(imail->mail.data_pool, 64);
 			mail_search_args_result_serialize(_ctx->args,
 				imail->data.search_results);
 		}
