@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2012 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2002-2013 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "array.h"
@@ -30,7 +30,7 @@ struct list_dir_context {
 	enum mailbox_info_flags info_flags;
 
 	/* all files in this directory */
-	ARRAY_DEFINE(entries, struct list_dir_entry);
+	ARRAY(struct list_dir_entry) entries;
 	unsigned int entry_idx;
 };
 
@@ -39,7 +39,7 @@ struct fs_list_iterate_context {
 
 	const char *const *valid_patterns;
 	/* roots can be either /foo, ~user/bar or baz */
-	ARRAY_DEFINE(roots, const char *);
+	ARRAY(const char *) roots;
 	unsigned int root_idx;
 	char sep;
 
@@ -99,8 +99,8 @@ fs_list_rename_invalid(struct fs_list_iterate_context *ctx,
 	string_t *dest = t_str_new(128);
 	const char *root, *src;
 
-	root = mailbox_list_get_path(ctx->ctx.list, NULL,
-				     MAILBOX_LIST_PATH_TYPE_MAILBOX);
+	root = mailbox_list_get_root_forced(ctx->ctx.list,
+					  MAILBOX_LIST_PATH_TYPE_MAILBOX);
 	src = t_strconcat(root, "/", storage_name, NULL);
 
 	(void)uni_utf8_get_valid_data((const void *)storage_name,
@@ -155,8 +155,8 @@ dir_entry_get(struct fs_list_iterate_context *ctx, const char *dir_path,
 	}
 	if (strcmp(d->d_name, ctx->ctx.list->set.subscription_fname) == 0) {
 		/* if this is the subscriptions file, skip it */
-		root_dir = mailbox_list_get_path(ctx->ctx.list, NULL,
-						 MAILBOX_LIST_PATH_TYPE_DIR);
+		root_dir = mailbox_list_get_root_forced(ctx->ctx.list,
+							MAILBOX_LIST_PATH_TYPE_DIR);
 		if (strcmp(root_dir, dir_path) == 0)
 			return 0;
 	}
@@ -237,8 +237,10 @@ fs_list_get_storage_path(struct fs_list_iterate_context *ctx,
 	}
 	if (*path != '/') {
 		/* non-absolute path. add the mailbox root dir as prefix. */
-		root = mailbox_list_get_path(ctx->ctx.list, NULL,
-					     MAILBOX_LIST_PATH_TYPE_MAILBOX);
+		if (!mailbox_list_get_root_path(ctx->ctx.list,
+						MAILBOX_LIST_PATH_TYPE_MAILBOX,
+						&root))
+			return FALSE;
 		path = *path == '\0' ? root :
 			t_strconcat(root, "/", path, NULL);
 	}
@@ -252,8 +254,7 @@ fs_list_dir_read(struct fs_list_iterate_context *ctx,
 {
 	DIR *fsdir;
 	struct dirent *d;
-	struct list_dir_entry *entry;
-	const char *path, *vname;
+	const char *path;
 	int ret = 0;
 
 	if (!fs_list_get_storage_path(ctx, dir->storage_name, &path))
@@ -301,20 +302,6 @@ fs_list_dir_read(struct fs_list_iterate_context *ctx,
 			"closedir(%s) failed: %m", path);
 		ret = -1;
 	}
-	if ((ctx->ctx.flags & MAILBOX_LIST_ITER_SHOW_EXISTING_PARENT) != 0 &&
-	    ctx->dir == NULL && *dir->storage_name != '\0') {
-		/* LIST "" foo/% shows foo/ if it exists */
-		vname = mailbox_list_get_vname(ctx->ctx.list,
-					       dir->storage_name);
-		vname = t_strdup_printf("%s%c", vname, ctx->sep);
-		if (imap_match(ctx->ctx.glob, vname) == IMAP_MATCH_YES) {
-			entry = array_append_space(&dir->entries);
-			entry->fname = "";
-			entry->info_flags = MAILBOX_NOSELECT |
-				(dir->info_flags &
-				 (MAILBOX_CHILDREN | MAILBOX_NOCHILDREN));
-		}
-	}
 	return ret;
 }
 
@@ -348,8 +335,8 @@ fs_list_get_valid_patterns(struct fs_list_iterate_context *ctx,
 			   const char *const *patterns)
 {
 	struct mailbox_list *_list = ctx->ctx.list;
-	ARRAY_DEFINE(valid_patterns, const char *);
-	const char *pattern, *test_pattern, *real_pattern;
+	ARRAY(const char *) valid_patterns;
+	const char *pattern, *test_pattern, *real_pattern, *error;
 	unsigned int prefix_len;
 
 	prefix_len = strlen(_list->ns->prefix);
@@ -366,13 +353,13 @@ fs_list_get_valid_patterns(struct fs_list_iterate_context *ctx,
 		   separators. */
 		real_pattern =
 			mailbox_list_get_storage_name(_list, test_pattern);
-		if (mailbox_list_is_valid_pattern(_list, test_pattern) &&
-		    mailbox_list_is_valid_pattern(_list, real_pattern)) {
+		if (mailbox_list_is_valid_name(_list, test_pattern, &error) &&
+		    mailbox_list_is_valid_name(_list, real_pattern, &error)) {
 			pattern = p_strdup(ctx->ctx.pool, *patterns);
 			array_append(&valid_patterns, &pattern, 1);
 		}
 	}
-	(void)array_append_space(&valid_patterns); /* NULL-terminate */
+	array_append_zero(&valid_patterns); /* NULL-terminate */
 	ctx->valid_patterns = array_idx(&valid_patterns, 0);
 
 	return array_count(&valid_patterns) > 1;
@@ -403,7 +390,6 @@ static void fs_list_get_roots(struct fs_list_iterate_context *ctx)
 			   where we could optimize with root=prefix, but
 			   probably too much trouble to implement. */
 			prefix_vname = "";
-			p = last = pattern;
 		} else {
 			for (p = last = pattern; *p != '\0'; p++) {
 				if (*p == '%' || *p == '*')
@@ -414,9 +400,12 @@ static void fs_list_get_roots(struct fs_list_iterate_context *ctx)
 			prefix_vname = t_strdup_until(pattern, last);
 		}
 
-		if (p == last+1 && *pattern == ns_sep)
+		if (*pattern == ns_sep && full_fs_access) {
+			/* pattern=/something with full filesystem access.
+			   (without full filesystem access we want to skip this
+			   if namespace prefix begins with separator) */
 			root = "/";
-		else if ((ns->flags & NAMESPACE_FLAG_INBOX_USER) != 0 &&
+		} else if ((ns->flags & NAMESPACE_FLAG_INBOX_USER) != 0 &&
 			 ns->prefix_len == 6 &&
 			 strcasecmp(prefix_vname, "INBOX") == 0 &&
 			 strncasecmp(ns->prefix, pattern, ns->prefix_len) == 0) {
@@ -425,7 +414,7 @@ static void fs_list_get_roots(struct fs_list_iterate_context *ctx)
 			   INBOX's children). */
 			root = "";
 		} else if ((ns->flags & NAMESPACE_FLAG_INBOX_ANY) != 0 &&
-			   ns->type == NAMESPACE_SHARED &&
+			   ns->type == MAIL_NAMESPACE_TYPE_SHARED &&
 			   !ctx->ctx.list->mail_set->mail_shared_explicit_inbox &&
 			   (prefix_vname[0] == '\0' ||
 			    (strncmp(ns->prefix, prefix_vname, ns->prefix_len-1) == 0 &&
@@ -595,7 +584,7 @@ static bool
 list_file_unfound_inbox(struct fs_list_iterate_context *ctx)
 {
 	ctx->info.flags = 0;
-	ctx->info.name = fs_list_get_inbox_vname(ctx);
+	ctx->info.vname = fs_list_get_inbox_vname(ctx);
 
 	if (mailbox_list_mailbox(ctx->ctx.list, "INBOX", &ctx->info.flags) < 0)
 		ctx->ctx.failed = TRUE;
@@ -624,8 +613,9 @@ list_file_is_any_inbox(struct fs_list_iterate_context *ctx,
 	if (!fs_list_get_storage_path(ctx, storage_name, &path))
 		return FALSE;
 
-	inbox_path = mailbox_list_get_path(ctx->ctx.list, "INBOX",
-					   MAILBOX_LIST_PATH_TYPE_DIR);
+	if (mailbox_list_get_path(ctx->ctx.list, "INBOX",
+				  MAILBOX_LIST_PATH_TYPE_DIR, &inbox_path) <= 0)
+		i_unreached();
 	return strcmp(path, inbox_path) == 0;
 }
 
@@ -642,12 +632,12 @@ fs_list_entry(struct fs_list_iterate_context *ctx,
 	storage_name = dir_get_storage_name(dir, entry->fname);
 
 	vname = mailbox_list_get_vname(ctx->ctx.list, storage_name);
-	ctx->info.name = p_strdup(ctx->info_pool, vname);
+	ctx->info.vname = p_strdup(ctx->info_pool, vname);
 	ctx->info.flags = entry->info_flags;
 
-	match = imap_match(ctx->ctx.glob, ctx->info.name);
+	match = imap_match(ctx->ctx.glob, ctx->info.vname);
 
-	child_dir_name = t_strdup_printf("%s%c", ctx->info.name, ctx->sep);
+	child_dir_name = t_strdup_printf("%s%c", ctx->info.vname, ctx->sep);
 	child_dir_match = imap_match(ctx->ctx.glob, child_dir_name);
 	if (child_dir_match == IMAP_MATCH_YES)
 		child_dir_match |= IMAP_MATCH_CHILDREN;
@@ -672,7 +662,7 @@ fs_list_entry(struct fs_list_iterate_context *ctx,
 	}
 
 	/* handle INBOXes correctly */
-	if (strcasecmp(ctx->info.name, "INBOX") == 0 &&
+	if (strcasecmp(ctx->info.vname, "INBOX") == 0 &&
 	    (ns->flags & NAMESPACE_FLAG_INBOX_USER) != 0) {
 		/* either this is user's INBOX, or it's a naming conflict */
 		if (!list_file_is_any_inbox(ctx, storage_name)) {
@@ -756,11 +746,11 @@ fs_list_next(struct fs_list_iterate_context *ctx)
 
 	if (ctx->list_inbox_inbox) {
 		ctx->info.flags = MAILBOX_CHILDREN | MAILBOX_NOSELECT;
-		ctx->info.name =
+		ctx->info.vname =
 			p_strconcat(ctx->info_pool,
 				    ctx->ctx.list->ns->prefix, "INBOX", NULL);
 		ctx->list_inbox_inbox = FALSE;
-		if (imap_match(ctx->ctx.glob, ctx->info.name) == IMAP_MATCH_YES)
+		if (imap_match(ctx->ctx.glob, ctx->info.vname) == IMAP_MATCH_YES)
 			return 1;
 	}
 	if (!ctx->inbox_found && ctx->ctx.glob != NULL &&
@@ -794,18 +784,18 @@ fs_list_iter_next(struct mailbox_list_iterate_context *_ctx)
 	if (ret <= 0)
 		return NULL;
 
-	if (_ctx->list->ns->type == NAMESPACE_SHARED &&
+	if (_ctx->list->ns->type == MAIL_NAMESPACE_TYPE_SHARED &&
 	    !_ctx->list->ns->list->mail_set->mail_shared_explicit_inbox &&
-	    strlen(ctx->info.name) < _ctx->list->ns->prefix_len) {
+	    strlen(ctx->info.vname) < _ctx->list->ns->prefix_len) {
 		/* shared/user INBOX, IMAP code already lists it */
 		return fs_list_iter_next(_ctx);
 	}
 
 	if ((ctx->ctx.flags & MAILBOX_LIST_ITER_RETURN_SUBSCRIBED) != 0) {
 		mailbox_list_set_subscription_flags(ctx->ctx.list,
-						    ctx->info.name,
+						    ctx->info.vname,
 						    &ctx->info.flags);
 	}
-	i_assert(ctx->info.name != NULL);
+	i_assert(ctx->info.vname != NULL);
 	return &ctx->info;
 }
