@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2012 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2002-2013 Dovecot authors, see the included COPYING file */
 
 #include "login-common.h"
 #include "base64.h"
@@ -77,7 +77,7 @@ client_get_auth_flags(struct client *client)
 	return auth_flags;
 }
 
-static void
+static void ATTR_NULL(3, 4)
 call_client_callback(struct client *client, enum sasl_server_reply reply,
 		     const char *data, const char *const *args)
 {
@@ -154,7 +154,8 @@ static void master_send_request(struct anvil_request *anvil_request)
 			    master_auth_callback, client, &client->master_tag);
 }
 
-static void anvil_lookup_callback(const char *reply, void *context)
+static void ATTR_NULL(1)
+anvil_lookup_callback(const char *reply, void *context)
 {
 	struct anvil_request *req = context;
 	struct client *client = req->client;
@@ -188,7 +189,7 @@ anvil_check_too_many_connections(struct client *client,
 	req->auth_pid = auth_client_request_get_server_pid(request);
 	req->auth_id = auth_client_request_get_id(request);
 
-	buffer_create_data(&buf, req->cookie, sizeof(req->cookie));
+	buffer_create_from_data(&buf, req->cookie, sizeof(req->cookie));
 	cookie = auth_client_request_get_cookie(request);
 	if (strlen(cookie) == MASTER_AUTH_COOKIE_SIZE*2)
 		(void)hex_to_binary(cookie, &buf);
@@ -326,6 +327,10 @@ void sasl_server_auth_begin(struct client *client,
 	info.remote_ip = client->ip;
 	info.local_port = client->local_port;
 	info.remote_port = client->remote_port;
+	info.real_local_ip = client->real_local_ip;
+	info.real_remote_ip = client->real_remote_ip;
+	info.real_local_port = client->real_local_port;
+	info.real_remote_port = client->real_remote_port;
 	info.initial_resp_base64 = initial_resp_base64;
 
 	client->auth_request =
@@ -333,8 +338,9 @@ void sasl_server_auth_begin(struct client *client,
 					authenticate_callback, client);
 }
 
-static void sasl_server_auth_cancel(struct client *client, const char *reason,
-				    enum sasl_server_reply reply)
+static void ATTR_NULL(2)
+sasl_server_auth_cancel(struct client *client, const char *reason,
+			enum sasl_server_reply reply)
 {
 	i_assert(client->authenticating);
 
