@@ -1,15 +1,16 @@
-/* Copyright (c) 2002-2012 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2002-2013 Dovecot authors, see the included COPYING file */
 
 #include "login-common.h"
 #include "array.h"
 #include "ioloop.h"
-#include "network.h"
+#include "net.h"
 #include "ostream.h"
 #include "read-full.h"
 #include "safe-memset.h"
 #include "hash.h"
 #include "llist.h"
 #include "master-interface.h"
+#include "master-service-ssl-settings.h"
 #include "client-common.h"
 #include "ssl-proxy.h"
 
@@ -51,7 +52,8 @@ struct ssl_proxy {
 	SSL *ssl;
 	struct client *client;
 	struct ip_addr ip;
-	const struct login_settings *set;
+	const struct login_settings *login_set;
+	const struct master_service_ssl_settings *ssl_set;
 	pool_t set_pool;
 
 	int fd_ssl, fd_plain;
@@ -96,7 +98,8 @@ struct ssl_server_context {
 };
 
 static int extdata_index;
-static struct hash_table *ssl_servers;
+static HASH_TABLE(struct ssl_server_context *,
+		  struct ssl_server_context *) ssl_servers;
 static SSL_CTX *ssl_client_ctx;
 static unsigned int ssl_proxy_count;
 static struct ssl_proxy *ssl_proxies;
@@ -112,12 +115,12 @@ static void ssl_proxy_destroy(struct ssl_proxy *proxy);
 static void ssl_proxy_unref(struct ssl_proxy *proxy);
 
 static struct ssl_server_context *
-ssl_server_context_init(const struct login_settings *set);
+ssl_server_context_init(const struct login_settings *login_set,
+			const struct master_service_ssl_settings *ssl_set);
 static void ssl_server_context_deinit(struct ssl_server_context **_ctx);
 
-static unsigned int ssl_server_context_hash(const void *p)
+static unsigned int ssl_server_context_hash(const struct ssl_server_context *ctx)
 {
-	const struct ssl_server_context *ctx = p;
 	unsigned int i, g, h = 0;
 
 	/* checking for different certs is typically good enough,
@@ -132,10 +135,9 @@ static unsigned int ssl_server_context_hash(const void *p)
 	return h;
 }
 
-static int ssl_server_context_cmp(const void *p1, const void *p2)
+static int ssl_server_context_cmp(const struct ssl_server_context *ctx1,
+				  const struct ssl_server_context *ctx2)
 {
-	const struct ssl_server_context *ctx1 = p1, *ctx2 = p2;
-
 	if (strcmp(ctx1->cert, ctx2->cert) != 0)
 		return 1;
 	if (strcmp(ctx1->key, ctx2->key) != 0)
@@ -152,8 +154,7 @@ static int ssl_server_context_cmp(const void *p1, const void *p2)
 
 static void ssl_params_corrupted(void)
 {
-	i_fatal("Corrupted SSL parameters file: "
-		PKG_STATEDIR"/ssl-parameters.dat");
+	i_fatal("Corrupted SSL parameters file in state_dir: ssl-parameters.dat");
 }
 
 static void read_next(struct ssl_parameters *params, void *data, size_t size)
@@ -311,8 +312,8 @@ static void plain_read(struct ssl_proxy *proxy)
 		} else {
 			proxy->sslout_size += ret;
 			if (!corked) {
-				net_set_cork(proxy->fd_ssl, TRUE);
-				corked = TRUE;
+				if (net_set_cork(proxy->fd_ssl, TRUE) == 0)
+					corked = TRUE;
 			}
 			ssl_write(proxy);
 		}
@@ -548,7 +549,8 @@ static void ssl_step(struct ssl_proxy *proxy)
 
 static int
 ssl_proxy_alloc_common(SSL_CTX *ssl_ctx, int fd, const struct ip_addr *ip,
-		       pool_t set_pool, const struct login_settings *set,
+		       pool_t set_pool, const struct login_settings *login_set,
+		       const struct master_service_ssl_settings *ssl_set,
 		       struct ssl_proxy **proxy_r)
 {
 	struct ssl_proxy *proxy;
@@ -591,7 +593,8 @@ ssl_proxy_alloc_common(SSL_CTX *ssl_ctx, int fd, const struct ip_addr *ip,
 	proxy = i_new(struct ssl_proxy, 1);
 	proxy->refcount = 2;
 	proxy->ssl = ssl;
-	proxy->set = set;
+	proxy->login_set = login_set;
+	proxy->ssl_set = ssl_set;
 	proxy->fd_ssl = fd;
 	proxy->fd_plain = sfd[0];
 	proxy->ip = *ip;
@@ -607,7 +610,8 @@ ssl_proxy_alloc_common(SSL_CTX *ssl_ctx, int fd, const struct ip_addr *ip,
 }
 
 static struct ssl_server_context *
-ssl_server_context_get(const struct login_settings *set)
+ssl_server_context_get(const struct login_settings *login_set,
+		       const struct master_service_ssl_settings *set)
 {
 	struct ssl_server_context *ctx, lookup_ctx;
 
@@ -617,34 +621,38 @@ ssl_server_context_get(const struct login_settings *set)
 	lookup_ctx.ca = set->ssl_ca;
 	lookup_ctx.cipher_list = set->ssl_cipher_list;
 	lookup_ctx.protocols = set->ssl_protocols;
-	lookup_ctx.verify_client_cert = set->ssl_verify_client_cert;
+	lookup_ctx.verify_client_cert = set->ssl_verify_client_cert ||
+		login_set->auth_ssl_require_client_cert ||
+		login_set->auth_ssl_username_from_cert;
 
 	ctx = hash_table_lookup(ssl_servers, &lookup_ctx);
 	if (ctx == NULL)
-		ctx = ssl_server_context_init(set);
+		ctx = ssl_server_context_init(login_set, set);
 	return ctx;
 }
 
 int ssl_proxy_alloc(int fd, const struct ip_addr *ip, pool_t set_pool,
-		    const struct login_settings *set,
+		    const struct login_settings *login_set,
+		    const struct master_service_ssl_settings *ssl_set,
 		    struct ssl_proxy **proxy_r)
 {
 	struct ssl_server_context *ctx;
 
-	ctx = ssl_server_context_get(set);
+	ctx = ssl_server_context_get(login_set, ssl_set);
 	return ssl_proxy_alloc_common(ctx->ctx, fd, ip,
-				      set_pool, set, proxy_r);
+				      set_pool, login_set, ssl_set, proxy_r);
 }
 
 int ssl_proxy_client_alloc(int fd, struct ip_addr *ip, pool_t set_pool,
-			   const struct login_settings *set,
+			   const struct login_settings *login_set,
+			   const struct master_service_ssl_settings *ssl_set,
 			   ssl_handshake_callback_t *callback, void *context,
 			   struct ssl_proxy **proxy_r)
 {
 	int ret;
 
 	ret = ssl_proxy_alloc_common(ssl_client_ctx, fd, ip,
-				     set_pool, set, proxy_r);
+				     set_pool, login_set, ssl_set, proxy_r);
 	if (ret < 0)
 		return -1;
 
@@ -803,8 +811,8 @@ static void ssl_proxy_destroy(struct ssl_proxy *proxy)
 
 	(void)SSL_shutdown(proxy->ssl);
 
-	(void)net_disconnect(proxy->fd_ssl);
-	(void)net_disconnect(proxy->fd_plain);
+	net_disconnect(proxy->fd_ssl);
+	net_disconnect(proxy->fd_plain);
 
 	if (proxy->client != NULL)
 		client_unref(&proxy->client);
@@ -834,7 +842,7 @@ static void ssl_info_callback(const SSL *ssl, int where, int ret)
 
 	proxy = SSL_get_ex_data(ssl, extdata_index);
 
-	if (!proxy->set->verbose_ssl)
+	if (!proxy->ssl_set->verbose_ssl)
 		return;
 
 	if ((where & SSL_CB_ALERT) != 0) {
@@ -865,7 +873,7 @@ static int ssl_verify_client_cert(int preverify_ok, X509_STORE_CTX *ctx)
 	proxy = SSL_get_ex_data(ssl, extdata_index);
 	proxy->cert_received = TRUE;
 
-	if (proxy->client_proxy && !proxy->set->ssl_require_crl &&
+	if (proxy->client_proxy && !proxy->login_set->ssl_require_crl &&
 	    (ctx->error == X509_V_ERR_UNABLE_TO_GET_CRL ||
 	     ctx->error == X509_V_ERR_CRL_HAS_EXPIRED)) {
 		/* no CRL given with the CA list. don't worry about it. */
@@ -883,8 +891,8 @@ static int ssl_verify_client_cert(int preverify_ok, X509_STORE_CTX *ctx)
 			X509_verify_cert_error_string(ctx->error), buf);
 	}
 
-	if (proxy->set->verbose_ssl ||
-	    (proxy->set->auth_verbose && !preverify_ok)) {
+	if (proxy->ssl_set->verbose_ssl ||
+	    (proxy->login_set->auth_verbose && !preverify_ok)) {
 		if (preverify_ok)
 			i_info("Valid certificate: %s", buf);
 		else {
@@ -965,7 +973,7 @@ static void load_ca(X509_STORE *store, const char *ca,
 }
 
 static STACK_OF(X509_NAME) *
-ssl_proxy_ctx_init(SSL_CTX *ssl_ctx, const struct login_settings *set,
+ssl_proxy_ctx_init(SSL_CTX *ssl_ctx, const struct master_service_ssl_settings *set,
 		   bool load_xnames)
 {
 	X509_STORE *store;
@@ -1038,7 +1046,7 @@ static const char *ssl_key_load_error(void)
 		return ssl_last_error();
 }
 
-static EVP_PKEY *
+static EVP_PKEY * ATTR_NULL(2)
 ssl_proxy_load_key(const char *key, const char *password)
 {
 	EVP_PKEY *pkey;
@@ -1060,7 +1068,9 @@ ssl_proxy_load_key(const char *key, const char *password)
 	return pkey;
 }
 
-static void ssl_proxy_ctx_use_key(SSL_CTX *ctx, const struct login_settings *set)
+static void
+ssl_proxy_ctx_use_key(SSL_CTX *ctx,
+		      const struct master_service_ssl_settings *set)
 {
 	EVP_PKEY *pkey;
 	const char *password;
@@ -1145,66 +1155,17 @@ static void ssl_servername_callback(SSL *ssl, int *al ATTR_UNUSED,
 		client->set = login_settings_read(client->pool,
 						  &client->local_ip,
 						  &client->ip, host,
+						  &client->ssl_set,
 						  &other_sets);
 	}
-	ctx = ssl_server_context_get(client->set);
+	ctx = ssl_server_context_get(client->set, client->ssl_set);
 	SSL_set_SSL_CTX(ssl, ctx->ctx);
 }
 #endif
 
-enum {
-	DOVECOT_SSL_PROTO_SSLv2	= 0x01,
-	DOVECOT_SSL_PROTO_SSLv3	= 0x02,
-	DOVECOT_SSL_PROTO_TLSv1	= 0x04,
-	DOVECOT_SSL_PROTO_ALL	= 0x07
-};
-
-static void
-ssl_proxy_ctx_set_protocols(struct ssl_server_context *ssl_ctx,
-			    const char *protocols)
-{
-	const char *const *tmp;
-	int proto, op = 0, include = 0, exclude = 0;
-	bool neg;
-
-	tmp = t_strsplit_spaces(protocols, " ");
-	for (; *tmp != NULL; tmp++) {
-		const char *name = *tmp;
-
-		if (*name != '!')
-			neg = FALSE;
-		else {
-			name++;
-			neg = TRUE;
-		}
-		if (strcasecmp(name, SSL_TXT_SSLV2) == 0)
-			proto = DOVECOT_SSL_PROTO_SSLv2;
-		else if (strcasecmp(name, SSL_TXT_SSLV3) == 0)
-			proto = DOVECOT_SSL_PROTO_SSLv3;
-		else if (strcasecmp(name, SSL_TXT_TLSV1) == 0)
-			proto = DOVECOT_SSL_PROTO_TLSv1;
-		else {
-			i_fatal("Invalid ssl_protocols setting: "
-				"Unknown protocol '%s'", name);
-		}
-		if (neg)
-			exclude |= proto;
-		else
-			include |= proto;
-	}
-	if (include != 0) {
-		/* exclude everything, except those that are included
-		   (and let excludes still override those) */
-		exclude |= DOVECOT_SSL_PROTO_ALL & ~include;
-	}
-	if ((exclude & DOVECOT_SSL_PROTO_SSLv2) != 0) op |= SSL_OP_NO_SSLv2;
-	if ((exclude & DOVECOT_SSL_PROTO_SSLv3) != 0) op |= SSL_OP_NO_SSLv3;
-	if ((exclude & DOVECOT_SSL_PROTO_TLSv1) != 0) op |= SSL_OP_NO_TLSv1;
-	SSL_CTX_set_options(ssl_ctx->ctx, op);
-}
-
 static struct ssl_server_context *
-ssl_server_context_init(const struct login_settings *set)
+ssl_server_context_init(const struct login_settings *login_set,
+			const struct master_service_ssl_settings *ssl_set)
 {
 	struct ssl_server_context *ctx;
 	SSL_CTX *ssl_ctx;
@@ -1214,23 +1175,25 @@ ssl_server_context_init(const struct login_settings *set)
 	pool = pool_alloconly_create("ssl server context", 4096);
 	ctx = p_new(pool, struct ssl_server_context, 1);
 	ctx->pool = pool;
-	ctx->cert = p_strdup(pool, set->ssl_cert);
-	ctx->key = p_strdup(pool, set->ssl_key);
-	ctx->ca = p_strdup(pool, set->ssl_ca);
-	ctx->cipher_list = p_strdup(pool, set->ssl_cipher_list);
-	ctx->protocols = p_strdup(pool, set->ssl_protocols);
-	ctx->verify_client_cert = set->ssl_verify_client_cert;
+	ctx->cert = p_strdup(pool, ssl_set->ssl_cert);
+	ctx->key = p_strdup(pool, ssl_set->ssl_key);
+	ctx->ca = p_strdup(pool, ssl_set->ssl_ca);
+	ctx->cipher_list = p_strdup(pool, ssl_set->ssl_cipher_list);
+	ctx->protocols = p_strdup(pool, ssl_set->ssl_protocols);
+	ctx->verify_client_cert = ssl_set->ssl_verify_client_cert ||
+		login_set->auth_ssl_require_client_cert ||
+		login_set->auth_ssl_username_from_cert;
 
 	ctx->ctx = ssl_ctx = SSL_CTX_new(SSLv23_server_method());
 	if (ssl_ctx == NULL)
 		i_fatal("SSL_CTX_new() failed");
-	xnames = ssl_proxy_ctx_init(ssl_ctx, set, ctx->verify_client_cert);
+	xnames = ssl_proxy_ctx_init(ssl_ctx, ssl_set, ctx->verify_client_cert);
 
 	if (SSL_CTX_set_cipher_list(ssl_ctx, ctx->cipher_list) != 1) {
 		i_fatal("Can't set cipher list to '%s': %s",
 			ctx->cipher_list, ssl_last_error());
 	}
-	ssl_proxy_ctx_set_protocols(ctx, ctx->protocols);
+	SSL_CTX_set_options(ssl_ctx, openssl_get_protocol_options(ctx->protocols));
 
 	if (ssl_proxy_ctx_use_certificate_chain(ctx->ctx, ctx->cert) != 1) {
 		i_fatal("Can't load ssl_cert: %s",
@@ -1240,12 +1203,12 @@ ssl_server_context_init(const struct login_settings *set)
 #ifdef HAVE_SSL_GET_SERVERNAME
 	if (SSL_CTX_set_tlsext_servername_callback(ctx->ctx,
 						   ssl_servername_callback) != 1) {
-		if (set->verbose_ssl)
+		if (ssl_set->verbose_ssl)
 			i_debug("OpenSSL library doesn't support SNI");
 	}
 #endif
 
-	ssl_proxy_ctx_use_key(ctx->ctx, set);
+	ssl_proxy_ctx_use_key(ctx->ctx, ssl_set);
 	SSL_CTX_set_info_callback(ctx->ctx, ssl_info_callback);
 
 	if (ctx->verify_client_cert)
@@ -1285,37 +1248,40 @@ ssl_proxy_client_ctx_set_client_cert(SSL_CTX *ctx,
 	EVP_PKEY_free(pkey);
 }
 
-static void ssl_proxy_init_client(const struct login_settings *set)
+static void
+ssl_proxy_init_client(const struct login_settings *login_set,
+		      const struct master_service_ssl_settings *ssl_set)
 {
 	STACK_OF(X509_NAME) *xnames;
 
 	if ((ssl_client_ctx = SSL_CTX_new(SSLv23_client_method())) == NULL)
 		i_fatal("SSL_CTX_new() failed");
-	xnames = ssl_proxy_ctx_init(ssl_client_ctx, set, TRUE);
+	xnames = ssl_proxy_ctx_init(ssl_client_ctx, ssl_set, TRUE);
 	ssl_proxy_ctx_verify_client(ssl_client_ctx, xnames);
 
-	ssl_proxy_client_ctx_set_client_cert(ssl_client_ctx, set);
+	ssl_proxy_client_ctx_set_client_cert(ssl_client_ctx, login_set);
 }
 
 void ssl_proxy_init(void)
 {
-	const struct login_settings *set = global_login_settings;
+	const struct login_settings *login_set = global_login_settings;
+	const struct master_service_ssl_settings *ssl_set = global_ssl_settings;
 	static char dovecot[] = "dovecot";
 	unsigned char buf;
 
-	if (strcmp(set->ssl, "no") == 0)
+	if (strcmp(ssl_set->ssl, "no") == 0)
 		return;
 
 	SSL_library_init();
 	SSL_load_error_strings();
 	OpenSSL_add_all_algorithms();
 
-	if (*set->ssl_crypto_device != '\0') {
+	if (*ssl_set->ssl_crypto_device != '\0') {
 		ENGINE_load_builtin_engines();
-		ssl_engine = ENGINE_by_id(set->ssl_crypto_device);
+		ssl_engine = ENGINE_by_id(ssl_set->ssl_crypto_device);
 		if (ssl_engine == NULL) {
 			i_fatal("Unknown ssl_crypto_device: %s",
-				set->ssl_crypto_device);
+				ssl_set->ssl_crypto_device);
 		}
 		ENGINE_init(ssl_engine);
 		ENGINE_set_default_RSA(ssl_engine);
@@ -1325,16 +1291,15 @@ void ssl_proxy_init(void)
 
 	extdata_index = SSL_get_ex_new_index(0, dovecot, NULL, NULL, NULL);
 
-	ssl_servers = hash_table_create(default_pool, default_pool, 0,
-					ssl_server_context_hash,
-					ssl_server_context_cmp);
-	(void)ssl_server_context_init(set);
+	hash_table_create(&ssl_servers, default_pool, 0,
+			  ssl_server_context_hash, ssl_server_context_cmp);
+	(void)ssl_server_context_init(login_set, ssl_set);
 
-	ssl_proxy_init_client(set);
-	ssl_username_nid = OBJ_txt2nid(set->ssl_cert_username_field);
+	ssl_proxy_init_client(login_set, ssl_set);
+	ssl_username_nid = OBJ_txt2nid(ssl_set->ssl_cert_username_field);
 	if (ssl_username_nid == NID_undef) {
 		i_fatal("Invalid ssl_cert_username_field: %s",
-			set->ssl_cert_username_field);
+			ssl_set->ssl_cert_username_field);
 	}
 
 	/* PRNG initialization might want to use /dev/urandom, make sure it
@@ -1354,7 +1319,7 @@ void ssl_proxy_init(void)
 void ssl_proxy_deinit(void)
 {
 	struct hash_iterate_context *iter;
-	void *key, *value;
+	struct ssl_server_context *ctx;
 
 	if (!ssl_initialized)
 		return;
@@ -1363,11 +1328,8 @@ void ssl_proxy_deinit(void)
 		ssl_proxy_destroy(ssl_proxies);
 
 	iter = hash_table_iterate_init(ssl_servers);
-	while (hash_table_iterate(iter, &key, &value)) {
-		struct ssl_server_context *ctx = value;
-
+	while (hash_table_iterate(iter, ssl_servers, &ctx, &ctx))
 		ssl_server_context_deinit(&ctx);
-	}
 	hash_table_iterate_deinit(&iter);
 	hash_table_destroy(&ssl_servers);
 
