@@ -1,4 +1,4 @@
-/* Copyright (c) 2006-2012 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2006-2013 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "ioloop.h"
@@ -19,6 +19,7 @@
 #include <stdlib.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <utime.h>
 #include <sys/stat.h>
 
 #define ACL_ESTALE_RETRY_COUNT NFS_ESTALE_RETRY_COUNT
@@ -129,29 +130,36 @@ static const char *
 acl_backend_vfile_get_local_dir(struct acl_backend *backend, const char *name)
 {
 	struct mail_namespace *ns = mailbox_list_get_namespace(backend->list);
-	const char *dir, *inbox;
+	enum mailbox_list_path_type type;
+	const char *dir, *inbox, *error;
 
 	if (*name == '\0')
 		name = NULL;
-	else if (!mailbox_list_is_valid_existing_name(ns->list, name))
+	else if (!mailbox_list_is_valid_name(ns->list, name, &error))
 		return NULL;
 
-	if (mail_storage_is_mailbox_file(ns->storage)) {
-		dir = mailbox_list_get_path(ns->list, name,
-					    MAILBOX_LIST_PATH_TYPE_CONTROL);
+	/* ACL files are very important. try to keep them among the main
+	   mail files. that's not possible though with a) if the mailbox is
+	   a file or b) if the mailbox path doesn't point to filesystem. */
+	type = mail_storage_is_mailbox_file(ns->storage) ||
+		(ns->storage->class_flags & MAIL_STORAGE_CLASS_FLAG_NO_ROOT) != 0 ?
+		MAILBOX_LIST_PATH_TYPE_CONTROL : MAILBOX_LIST_PATH_TYPE_MAILBOX;
+	if (name == NULL) {
+		if (!mailbox_list_get_root_path(ns->list, type, &dir))
+			return FALSE;
 	} else {
-		dir = mailbox_list_get_path(ns->list, name,
-					    MAILBOX_LIST_PATH_TYPE_MAILBOX);
-	}
-	if (name == NULL && dir != NULL) {
-		/* verify that the directory isn't same as INBOX's directory.
-		   this is mainly for Maildir. */
-		inbox = mailbox_list_get_path(ns->list, "INBOX",
-					      MAILBOX_LIST_PATH_TYPE_MAILBOX);
-		if (strcmp(inbox, dir) == 0) {
-			/* can't have default ACLs with this setup */
+		if (mailbox_list_get_path(ns->list, name, type, &dir) <= 0)
 			return NULL;
-		}
+	}
+
+	/* verify that the directory isn't same as INBOX's directory.
+	   this is mainly for Maildir. */
+	if (name == NULL &&
+	    mailbox_list_get_path(ns->list, "INBOX",
+				  MAILBOX_LIST_PATH_TYPE_MAILBOX, &inbox) > 0 &&
+	    strcmp(inbox, dir) == 0) {
+		/* can't have default ACLs with this setup */
+		return NULL;
 	}
 	return dir;
 }
@@ -237,11 +245,13 @@ acl_backend_vfile_has_acl(struct acl_backend *_backend, const char *name)
 	/* See if the mailbox exists. If we wanted recursive lookups we could
 	   skip this, but at least for now we assume that if an existing
 	   mailbox has no ACL it's equivalent to default ACLs. */
-	path = mailbox_list_get_path(_backend->list, name,
-				     MAILBOX_LIST_PATH_TYPE_MAILBOX);
-	ret = path == NULL ? 0 :
-		acl_backend_vfile_exists(backend, path,
-					 &new_validity.mailbox_validity);
+	if (mailbox_list_get_path(_backend->list, name,
+				  MAILBOX_LIST_PATH_TYPE_MAILBOX, &path) <= 0)
+		ret = -1;
+	else {
+		ret = acl_backend_vfile_exists(backend, path,
+					       &new_validity.mailbox_validity);
+	}
 	if (ret == 0 &&
 	    (dir = acl_backend_vfile_get_local_dir(_backend, name)) != NULL) {
 		local_path = t_strconcat(dir, "/", name, NULL);
@@ -482,25 +492,25 @@ acl_backend_vfile_read(struct acl_object_vfile *aclobj,
 
 	if (fstat(fd, &st) < 0) {
 		if (errno == ESTALE && try_retry) {
-			(void)close(fd);
+			i_close_fd(&fd);
 			return 0;
 		}
 
 		i_error("fstat(%s) failed: %m", path);
-		(void)close(fd);
+		i_close_fd(&fd);
 		return -1;
 	}
 	if (S_ISDIR(st.st_mode)) {
 		/* we opened a directory. */
 		*is_dir_r = TRUE;
-		(void)close(fd);
+		i_close_fd(&fd);
 		return 0;
 	}
 
 	if (aclobj->aclobj.backend->debug)
 		i_debug("acl vfile: reading file %s", path);
 
-	input = i_stream_create_fd(fd, 4096, FALSE);
+	input = i_stream_create_fd(fd, (size_t)-1, FALSE);
 	i_stream_set_return_partial_line(input, TRUE);
 	linenum = 1;
 	while ((line = i_stream_read_next_line(input)) != NULL) {
@@ -987,6 +997,7 @@ vfile_object_add_right(struct acl_object_vfile *aclobj, unsigned int idx,
 		       const struct acl_rights_update *update)
 {
 	struct acl_rights right;
+	bool c1, c2;
 
 	if (update->modify_mode == ACL_MODIFY_MODE_REMOVE &&
 	    update->neg_modify_mode == ACL_MODIFY_MODE_REMOVE) {
@@ -998,8 +1009,17 @@ vfile_object_add_right(struct acl_object_vfile *aclobj, unsigned int idx,
 	right.id_type = update->rights.id_type;
 	right.identifier = p_strdup(aclobj->rights_pool,
 				    update->rights.identifier);
-	array_insert(&aclobj->rights, idx, &right, 1);
-	return vfile_object_modify_right(aclobj, idx, update);
+
+	c1 = modify_right_list(aclobj->rights_pool, &right.rights,
+			       update->rights.rights, update->modify_mode);
+	c2 = modify_right_list(aclobj->rights_pool, &right.neg_rights,
+			       update->rights.neg_rights,
+			       update->neg_modify_mode);
+	if (c1 || c2) {
+		array_insert(&aclobj->rights, idx, &right, 1);
+		return TRUE;
+	}
+	return FALSE;
 }
 
 static void vfile_write_rights_list(string_t *dest, const char *const *rights)
@@ -1072,17 +1092,17 @@ acl_backend_vfile_update_write(struct acl_object_vfile *aclobj,
 	for (i = 0; i < count && !rights[i].global; i++) {
 		if (rights[i].rights != NULL) {
 			vfile_write_right(str, &rights[i], FALSE);
-			o_stream_send(output, str_data(str), str_len(str));
+			o_stream_nsend(output, str_data(str), str_len(str));
 			str_truncate(str, 0);
 		}
 		if (rights[i].neg_rights != NULL) {
 			vfile_write_right(str, &rights[i], TRUE);
-			o_stream_send(output, str_data(str), str_len(str));
+			o_stream_nsend(output, str_data(str), str_len(str));
 			str_truncate(str, 0);
 		}
 	}
 	str_free(&str);
-	if (o_stream_flush(output) < 0) {
+	if (o_stream_nfinish(output) < 0) {
 		i_error("write(%s) failed: %m", path);
 		ret = -1;
 	}
@@ -1121,7 +1141,10 @@ acl_backend_vfile_object_update(struct acl_object *_aclobj,
 	struct acl_object_vfile *aclobj = (struct acl_object_vfile *)_aclobj;
 	struct acl_backend_vfile *backend =
 		(struct acl_backend_vfile *)_aclobj->backend;
+	struct acl_backend_vfile_validity *validity;
 	struct dotlock *dotlock;
+	struct utimbuf ut;
+	time_t orig_mtime;
 	const char *path;
 	unsigned int i;
 	int fd;
@@ -1144,6 +1167,10 @@ acl_backend_vfile_object_update(struct acl_object *_aclobj,
 		return 0;
 	}
 
+	validity = acl_cache_get_validity(_aclobj->backend->cache,
+					  _aclobj->name);
+	orig_mtime = validity->local_validity.last_mtime;
+
 	/* ACLs were really changed, write the new ones */
 	path = file_dotlock_get_lock_path(dotlock);
 	if (acl_backend_vfile_update_write(aclobj, fd, path) < 0) {
@@ -1151,6 +1178,16 @@ acl_backend_vfile_object_update(struct acl_object *_aclobj,
 		acl_cache_flush(_aclobj->backend->cache, _aclobj->name);
 		return -1;
 	}
+	if (orig_mtime < update->last_change && update->last_change != 0) {
+		/* set mtime to last_change, if it's higher than the file's
+		   original mtime. if original mtime is higher, then we're
+		   merging some changes and it's better for the mtime to get
+		   updated. */
+		ut.actime = ioloop_time;
+		ut.modtime = update->last_change;
+		if (utime(path, &ut) < 0)
+			i_error("utime(%s) failed: %m", path);
+	}
 	acl_backend_vfile_update_cache(_aclobj, fd);
 	if (file_dotlock_replace(&dotlock, 0) < 0) {
 		acl_cache_flush(_aclobj->backend->cache, _aclobj->name);
@@ -1165,6 +1202,27 @@ acl_backend_vfile_object_update(struct acl_object *_aclobj,
 	return 0;
 }
 
+static int acl_backend_vfile_object_last_changed(struct acl_object *_aclobj,
+						 time_t *last_changed_r)
+{
+	struct acl_backend_vfile_validity *old_validity;
+
+	*last_changed_r = 0;
+
+	old_validity = acl_cache_get_validity(_aclobj->backend->cache,
+					      _aclobj->name);
+	if (old_validity == NULL) {
+		if (acl_backend_vfile_object_refresh_cache(_aclobj) < 0)
+			return -1;
+		old_validity = acl_cache_get_validity(_aclobj->backend->cache,
+						      _aclobj->name);
+		if (old_validity == NULL)
+			return 0;
+	}
+	*last_changed_r = old_validity->local_validity.last_mtime;
+	return 0;
+}
+
 static struct acl_object_list_iter *
 acl_backend_vfile_object_list_init(struct acl_object *_aclobj)
 {
@@ -1221,6 +1279,7 @@ struct acl_backend_vfuncs acl_backend_vfile = {
 	acl_backend_vfile_object_deinit,
 	acl_backend_vfile_object_refresh_cache,
 	acl_backend_vfile_object_update,
+	acl_backend_vfile_object_last_changed,
 	acl_backend_vfile_object_list_init,
 	acl_backend_vfile_object_list_next,
 	acl_backend_vfile_object_list_deinit
