@@ -10,6 +10,7 @@ extern "C" {
 #include "mail-index.h"
 #include "mail-search.h"
 #include "mail-namespace.h"
+#include "mailbox-list-private.h"
 #include "mail-storage.h"
 #include "fts-expunge-log.h"
 #include "fts-lucene-plugin.h"
@@ -67,7 +68,7 @@ struct lucene_index {
 	IndexSearcher *searcher;
 
 	Analyzer *default_analyzer, *cur_analyzer;
-	ARRAY_DEFINE(analyzers, struct lucene_analyzer);
+	ARRAY(struct lucene_analyzer) analyzers;
 
 	Document *doc;
 	uint32_t prev_uid;
@@ -81,7 +82,7 @@ struct rescan_context {
 	int box_ret;
 
 	pool_t pool;
-	struct hash_table *guids;
+	HASH_TABLE(uint8_t *, uint8_t *) seen_mailbox_guids;
 
 	ARRAY_TYPE(seq_range) uids;
 	struct seq_range_iter uids_iter;
@@ -96,7 +97,7 @@ static bool textcat_broken = FALSE;
 static int textcat_refcount = 0;
 
 static void rescan_clear_unseen_mailboxes(struct lucene_index *index,
-					  struct hash_table *guids);
+					  struct rescan_context *rescan_ctx);
 
 struct lucene_index *lucene_index_init(const char *path,
 				       struct mailbox_list *list,
@@ -181,7 +182,7 @@ void lucene_utf8_n_to_tchar(const unsigned char *src, size_t srcsize,
 
 	i_assert(sizeof(wchar_t) == sizeof(unichar_t));
 
-	buffer_create_data(&buf, dest, sizeof(wchar_t) * destsize);
+	buffer_create_from_data(&buf, dest, sizeof(wchar_t) * destsize);
 	array_create_from_buffer(&dest_arr, &buf, sizeof(wchar_t));
 	if (uni_utf8_to_ucs4_n(src, srcsize, &dest_arr) < 0)
 		i_unreached();
@@ -237,7 +238,9 @@ static void lucene_handle_error(struct lucene_index *index, CLuceneError &err,
 	     err.number() == CL_ERR_IO)) {
 		/* delete corrupted index. most IO errors are also about
 		   missing files and other such corruption.. */
-		if (unlink_directory(index->path, TRUE) < 0 && errno != ENOENT)
+		if (unlink_directory(index->path,
+				     UNLINK_DIRECTORY_FLAG_RMDIR) < 0 &&
+		    errno != ENOENT)
 			i_error("unlink_directory(%s) failed: %m", index->path);
 		rescan_clear_unseen_mailboxes(index, NULL);
 	}
@@ -360,7 +363,7 @@ static int lucene_settings_check(struct lucene_index *index)
 		return ret;
 
 	/* settings changed, rebuild index */
-	if (unlink_directory(index->path, TRUE) < 0) {
+	if (unlink_directory(index->path, UNLINK_DIRECTORY_FLAG_RMDIR) < 0) {
 		i_error("unlink_directory(%s) failed: %m", index->path);
 		ret = -1;
 	} else {
@@ -586,7 +589,7 @@ wcharguid_to_guid(guid_128_t dest, const wchar_t *src)
 		return -1;
 	src_chars[i] = '\0';
 
-	buffer_create_data(&buf, dest, GUID_128_SIZE);
+	buffer_create_from_data(&buf, dest, GUID_128_SIZE);
 	return hex_to_binary(src_chars, &buf);
 }
 
@@ -654,7 +657,7 @@ rescan_open_mailbox(struct rescan_context *ctx, Document *doc)
 
 	guidp = p_new(ctx->pool, guid_128_t, 1);
 	memcpy(guidp, guid, sizeof(*guidp));
-	hash_table_insert(ctx->guids, guidp, guidp);
+	hash_table_insert(ctx->seen_mailbox_guids, guidp, guidp);
 
 	if (ctx->box != NULL)
 		rescan_finish(ctx);
@@ -734,7 +737,7 @@ rescan_next(struct rescan_context *ctx, Document *doc)
 }
 
 static void rescan_clear_unseen_mailboxes(struct lucene_index *index,
-					  struct hash_table *guids)
+					  struct rescan_context *rescan_ctx)
 {
 	const enum mailbox_list_iter_flags iter_flags =
 		(enum mailbox_list_iter_flags)
@@ -751,13 +754,14 @@ static void rescan_clear_unseen_mailboxes(struct lucene_index *index,
 
 	iter = mailbox_list_iter_init(index->list, "*", iter_flags);
 	while ((info = mailbox_list_iter_next(iter)) != NULL) {
-		box = mailbox_alloc(index->list, info->name,
+		box = mailbox_alloc(index->list, info->vname,
 				    (enum mailbox_flags)0);
 		if (mailbox_open(box) == 0 &&
 		    mailbox_get_metadata(box, MAILBOX_METADATA_GUID,
 					 &metadata) == 0 &&
-		    (guids == NULL ||
-		     hash_table_lookup(guids, metadata.guid) == NULL)) {
+		    (rescan_ctx == NULL ||
+		     hash_table_lookup(rescan_ctx->seen_mailbox_guids,
+				       metadata.guid) == NULL)) {
 			/* this mailbox had no records in lucene index.
 			   make sure its last indexed uid is 0 */
 			(void)fts_index_set_header(box, &hdr);
@@ -787,8 +791,8 @@ int lucene_index_rescan(struct lucene_index *index)
 	memset(&ctx, 0, sizeof(ctx));
 	ctx.index = index;
 	ctx.pool = pool_alloconly_create("guids", 1024);
-	ctx.guids = hash_table_create(default_pool, ctx.pool, 0,
-				      guid_128_hash, guid_128_cmp);
+	hash_table_create(&ctx.seen_mailbox_guids, ctx.pool, 0,
+			  guid_128_hash, guid_128_cmp);
 	i_array_init(&ctx.uids, 128);
 
 	if (ret > 0) try {
@@ -814,8 +818,8 @@ int lucene_index_rescan(struct lucene_index *index)
 		rescan_finish(&ctx);
 	array_free(&ctx.uids);
 
-	rescan_clear_unseen_mailboxes(index, ctx.guids);
-	hash_table_destroy(&ctx.guids);
+	rescan_clear_unseen_mailboxes(index, &ctx);
+	hash_table_destroy(&ctx.seen_mailbox_guids);
 	pool_unref(&ctx.pool);
 	return failed ? -1 : 0;
 }
@@ -827,7 +831,7 @@ static void guid128_to_wguid(const guid_128_t guid,
 	unsigned char guid_hex[MAILBOX_GUID_HEX_LENGTH];
 	unsigned int i;
 
-	buffer_create_data(&buf, guid_hex, MAILBOX_GUID_HEX_LENGTH);
+	buffer_create_from_data(&buf, guid_hex, MAILBOX_GUID_HEX_LENGTH);
 	binary_to_hex_append(&buf, guid, GUID_128_SIZE);
 	for (i = 0; i < MAILBOX_GUID_HEX_LENGTH; i++)
 		wguid_hex[i] = guid_hex[i];
@@ -1249,7 +1253,7 @@ lucene_index_search(struct lucene_index *index,
 				score->uid = uid;
 				score->score = hits->score(i);
 			}
-			seq_range_array_add(uids_r, 0, uid);
+			seq_range_array_add(uids_r, uid);
 		}
 		_CLDELETE(hits);
 		return ret;
@@ -1305,7 +1309,8 @@ int lucene_index_lookup(struct lucene_index *index,
 }
 
 static int
-lucene_index_search_multi(struct lucene_index *index, struct hash_table *guids,
+lucene_index_search_multi(struct lucene_index *index,
+			  HASH_TABLE_TYPE(wguid_result) guids,
 			  ARRAY_TYPE(lucene_query) &queries,
 			  struct fts_multi_result *result)
 {
@@ -1317,10 +1322,11 @@ lucene_index_search_multi(struct lucene_index *index, struct hash_table *guids,
 
 	BooleanQuery mailbox_query;
 	struct hash_iterate_context *iter;
-	void *key, *value;
+	wchar_t *key;
+	struct fts_result *value;
 	iter = hash_table_iterate_init(guids);
-	while (hash_table_iterate(iter, &key, &value)) {
-		Term *term = _CLNEW Term(_T("box"), (wchar_t *)key);
+	while (hash_table_iterate(iter, guids, &key, &value)) {
+		Term *term = _CLNEW Term(_T("box"), key);
 		TermQuery *q = _CLNEW TermQuery(term);
 		mailbox_query.add(q, true, BooleanClause::SHOULD);
 	}
@@ -1341,8 +1347,8 @@ lucene_index_search_multi(struct lucene_index *index, struct hash_table *guids,
 				ret = -1;
 				break;
 			}
-			struct fts_result *br = (struct fts_result *)
-				hash_table_lookup(guids, (const void *)box_guid);
+			struct fts_result *br =
+				hash_table_lookup(guids, box_guid);
 			if (br == NULL) {
 				i_warning("lucene: Returned unexpected mailbox with GUID %ls", box_guid);
 				continue;
@@ -1358,7 +1364,7 @@ lucene_index_search_multi(struct lucene_index *index, struct hash_table *guids,
 				p_array_init(&br->definite_uids, result->pool, 32);
 				p_array_init(&br->scores, result->pool, 32);
 			}
-			seq_range_array_add(&br->definite_uids, 0, uid);
+			seq_range_array_add(&br->definite_uids, uid);
 			score = array_append_space(&br->scores);
 			score->uid = uid;
 			score->score = hits->score(i);
@@ -1372,7 +1378,7 @@ lucene_index_search_multi(struct lucene_index *index, struct hash_table *guids,
 }
 
 int lucene_index_lookup_multi(struct lucene_index *index,
-			      struct hash_table *guids,
+			      HASH_TABLE_TYPE(wguid_result) guids,
 			      struct mail_search_arg *args, bool and_args,
 			      struct fts_multi_result *result)
 {
