@@ -1,4 +1,4 @@
-/* Copyright (c) 2006-2012 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2006-2013 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "array.h"
@@ -6,6 +6,7 @@
 #include "hash.h"
 #include "strescape.h"
 #include "unichar.h"
+#include "http-url.h"
 #include "imap-utf7.h"
 #include "mail-storage-private.h"
 #include "mailbox-list-private.h"
@@ -40,8 +41,6 @@ struct solr_fts_backend_update_context {
 	bool documents_added;
 };
 
-static struct solr_connection *solr_conn = NULL;
-
 static bool is_valid_xml_char(unichar_t chr)
 {
 	/* Valid characters in XML:
@@ -151,7 +150,7 @@ static void solr_quote(string_t *dest, const char *str)
 static void solr_quote_http(string_t *dest, const char *str)
 {
 	str_append(dest, "%22");
-	solr_connection_http_escape(solr_conn, dest, str);
+	http_url_escape_param(dest, str);
 	str_append(dest, "%22");
 }
 
@@ -226,16 +225,18 @@ static struct fts_backend *fts_backend_solr_alloc(void)
 }
 
 static int
-fts_backend_solr_init(struct fts_backend *_backend,
-		      const char **error_r ATTR_UNUSED)
+fts_backend_solr_init(struct fts_backend *_backend, const char **error_r)
 {
 	struct solr_fts_backend *backend = (struct solr_fts_backend *)_backend;
 	struct fts_solr_user *fuser = FTS_SOLR_USER_CONTEXT(_backend->ns->user);
 	const struct fts_solr_settings *set = &fuser->set;
 	const char *str;
 
-	if (solr_conn == NULL)
-		solr_conn = solr_connection_init(set->url, set->debug);
+	if (solr_conn == NULL) {
+		if (solr_connection_init(set->url, set->debug,
+					 &solr_conn, error_r) < 0)
+			return -1;
+	}
 
 	str = solr_escape_id_str(_backend->ns->user->username);
 	backend->id_username = i_strdup(str);
@@ -280,7 +281,7 @@ solr_add_ns_query_http(string_t *str, struct solr_fts_backend *backend,
 
 	tmp = t_str_new(64);
 	solr_add_ns_query(tmp, backend, ns, FALSE);
-	solr_connection_http_escape(solr_conn, str, str_c(tmp));
+	http_url_escape_param(str, str_c(tmp));
 }
 
 static int
@@ -349,7 +350,7 @@ fts_backend_solr_get_last_uid(struct fts_backend *_backend,
 	if (fts_backend_solr_get_last_uid_fallback(backend, box, last_uid_r) < 0)
 		return -1;
 
-	(void)fts_index_set_last_uid(box, *last_uid_r);
+	fts_index_set_last_uid(box, *last_uid_r);
 	return 0;
 }
 
@@ -468,7 +469,7 @@ fts_backend_solr_update_set_mailbox(struct fts_backend_update_context *_ctx,
 	struct mail_namespace *ns;
 
 	if (ctx->prev_uid != 0) {
-		(void)fts_index_set_last_uid(ctx->cur_box, ctx->prev_uid);
+		fts_index_set_last_uid(ctx->cur_box, ctx->prev_uid);
 		ctx->prev_uid = 0;
 	}
 
@@ -721,10 +722,10 @@ solr_search_multi(struct solr_fts_backend *backend, string_t *str,
 {
 	struct solr_result **solr_results;
 	struct fts_result *fts_result;
-	ARRAY_DEFINE(fts_results, struct fts_result);
+	ARRAY(struct fts_result) fts_results;
 	struct mail_namespace *ns;
 	struct mailbox_status status;
-	struct hash_table *mailboxes;
+	HASH_TABLE(char *, struct mailbox *) mailboxes;
 	struct mailbox *box;
 	const char *box_name;
 	char *box_id;
@@ -738,8 +739,7 @@ solr_search_multi(struct solr_fts_backend *backend, string_t *str,
 	else
 		str_append(str, "%22%22");
 
-	mailboxes = hash_table_create(default_pool, default_pool, 0,
-				      str_hash, (hash_cmp_callback_t *)strcmp);
+	hash_table_create(&mailboxes, default_pool, 0, str_hash, strcmp);
 	str_append(str, "%2B(");
 	len = str_len(str);
 	for (i = 0; boxes[i] != NULL; i++) {
@@ -778,7 +778,7 @@ solr_search_multi(struct solr_fts_backend *backend, string_t *str,
 		fts_result->scores = solr_results[i]->scores;
 		fts_result->scores_sorted = TRUE;
 	}
-	(void)array_append_space(&fts_results);
+	array_append_zero(&fts_results);
 	result->box_results = array_idx_modifiable(&fts_results, 0);
 	hash_table_destroy(&mailboxes);
 	return 0;
