@@ -1,4 +1,4 @@
-/* Copyright (c) 2006-2012 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2006-2013 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "array.h"
@@ -136,6 +136,26 @@ maildir_list_init(struct maildir_quota_root *root, struct mailbox_list *list)
 	return ctx;
 }
 
+static bool maildir_set_next_path(struct maildir_list_context *ctx)
+{
+	T_BEGIN {
+		const char *path, *storage_name;
+
+		str_truncate(ctx->path, 0);
+
+		storage_name = mailbox_list_get_storage_name(
+					ctx->info->ns->list, ctx->info->vname);
+		if (mailbox_list_get_path(ctx->list, storage_name,
+					  MAILBOX_LIST_PATH_TYPE_MAILBOX,
+					  &path) > 0) {
+			str_append(ctx->path, path);
+			str_append(ctx->path, ctx->state == 0 ?
+				   "/new" : "/cur");
+		}
+	} T_END;
+	return str_len(ctx->path) > 0;
+}
+
 static const char *
 maildir_list_next(struct maildir_list_context *ctx, time_t *mtime_r)
 {
@@ -149,25 +169,17 @@ maildir_list_next(struct maildir_list_context *ctx, time_t *mtime_r)
 				return NULL;
 
 			rule = quota_root_rule_find(ctx->root->root.set,
-						    ctx->info->name);
+						    ctx->info->vname);
 			if (rule != NULL && rule->ignore) {
 				/* mailbox not included in quota */
 				continue;
 			}
 		}
 
-		T_BEGIN {
-			const char *path, *storage_name;
-
-			storage_name = mailbox_list_get_storage_name(
-				ctx->info->ns->list, ctx->info->name);
-			path = mailbox_list_get_path(ctx->list, storage_name,
-					MAILBOX_LIST_PATH_TYPE_MAILBOX);
-			str_truncate(ctx->path, 0);
-			str_append(ctx->path, path);
-			str_append(ctx->path, ctx->state == 0 ?
-				   "/new" : "/cur");
-		} T_END;
+		if (!maildir_set_next_path(ctx)) {
+			ctx->state = 0;
+			continue;
+		}
 
 		if (++ctx->state == 2)
 			ctx->state = 0;
@@ -223,18 +235,18 @@ static int maildirsize_write(struct maildir_quota_root *root, const char *path)
 	struct mail_namespace *const *namespaces;
 	unsigned int i, count;
 	struct dotlock *dotlock;
-	const char *p, *dir, *gid_origin;
+	struct mailbox_permissions perm;
+	const char *p, *dir;
 	string_t *str;
-	mode_t file_mode, dir_mode;
-	gid_t gid;
 	int fd;
 
 	i_assert(root->fd == -1);
 
 	/* figure out what permissions we should use for maildirsize.
 	   use the inbox namespace's permissions if possible. */
-	file_mode = 0600; dir_mode = 0700; gid_origin = "default";
-	gid = (gid_t)-1;
+	perm.file_create_mode = 0600; perm.dir_create_mode = 0700;
+	perm.file_create_gid = (gid_t)-1;
+	perm.file_create_gid_origin = "default";
 	namespaces = array_get(&root->root.quota->namespaces, &count);
 	i_assert(count > 0);
 	for (i = 0; i < count; i++) {
@@ -242,8 +254,7 @@ static int maildirsize_write(struct maildir_quota_root *root, const char *path)
 			continue;
 
 		mailbox_list_get_root_permissions(namespaces[i]->list,
-						  &file_mode, &dir_mode,
-						  &gid, &gid_origin);
+						  &perm);
 		break;
 	}
 
@@ -251,19 +262,25 @@ static int maildirsize_write(struct maildir_quota_root *root, const char *path)
 	dotlock_settings.nfs_flush = set->mail_nfs_storage;
 	fd = file_dotlock_open_group(&dotlock_settings, path,
 				     DOTLOCK_CREATE_FLAG_NONBLOCK,
-				     file_mode, gid, gid_origin, &dotlock);
+				     perm.file_create_mode,
+				     perm.file_create_gid,
+				     perm.file_create_gid_origin, &dotlock);
 	if (fd == -1 && errno == ENOENT) {
 		/* the control directory doesn't exist yet? create it */
 		p = strrchr(path, '/');
 		dir = t_strdup_until(path, p);
-		if (mkdir_parents_chgrp(dir, dir_mode, gid, gid_origin) < 0 &&
+		if (mkdir_parents_chgrp(dir, perm.dir_create_mode,
+					perm.file_create_gid,
+					perm.file_create_gid_origin) < 0 &&
 		    errno != EEXIST) {
 			i_error("mkdir_parents(%s) failed: %m", dir);
 			return -1;
 		}
 		fd = file_dotlock_open_group(&dotlock_settings, path,
 					     DOTLOCK_CREATE_FLAG_NONBLOCK,
-					     file_mode, gid, gid_origin,
+					     perm.file_create_mode,
+					     perm.file_create_gid,
+					     perm.file_create_gid_origin,
 					     &dotlock);
 	}
 	if (fd == -1) {
@@ -593,8 +610,7 @@ static int maildirsize_read(struct maildir_quota_root *root, bool *retry)
 
 	if (ret < 0 && size == 0) {
 		/* the read failed and there's no usable header, fail. */
-		(void)close(root->fd);
-		root->fd = -1;
+		i_close_fd(&root->fd);
 		return -1;
 	}
 
@@ -610,8 +626,7 @@ static int maildirsize_read(struct maildir_quota_root *root, bool *retry)
 		ret = 1;
 	else {
 		/* broken file / need recalculation */
-		(void)close(root->fd);
-		root->fd = -1;
+		i_close_fd(&root->fd);
 		ret = 0;
 	}
 	return ret;
@@ -697,7 +712,7 @@ maildirquota_refresh(struct maildir_quota_root *root, bool *recalculated_r)
 static int maildirsize_update(struct maildir_quota_root *root,
 			      int count_diff, int64_t bytes_diff)
 {
-	char str[MAX_INT_STRLEN*2 + 2];
+	char str[MAX_INT_STRLEN*2 + 2 + 1];
 	int ret = 0;
 
 	if (count_diff == 0 && bytes_diff == 0)
@@ -708,8 +723,9 @@ static int maildirsize_update(struct maildir_quota_root *root,
 	   a while, and sooner if corruption causes calculations to go
 	   over quota. This is also how Maildir++ spec specifies it should be
 	   done.. */
-	i_snprintf(str, sizeof(str), "%lld %d\n",
-		   (long long)bytes_diff, count_diff);
+	if (i_snprintf(str, sizeof(str), "%lld %d\n",
+		       (long long)bytes_diff, count_diff) < 0)
+		i_unreached();
 	if (write_full(root->fd, str, strlen(str)) < 0) {
 		ret = -1;
 		if (errno == ESTALE) {
@@ -766,7 +782,7 @@ static void maildir_quota_deinit(struct quota_root *_root)
 	struct maildir_quota_root *root = (struct maildir_quota_root *)_root;
 
 	if (root->fd != -1)
-		(void)close(root->fd);
+		i_close_fd(&root->fd);
 	i_free(root);
 }
 
@@ -807,8 +823,9 @@ maildir_quota_root_namespace_added(struct quota_root *_root,
 	if (root->maildirsize_path != NULL)
 		return;
 
-	control_dir = mailbox_list_get_path(ns->list, NULL,
-					    MAILBOX_LIST_PATH_TYPE_CONTROL);
+	if (!mailbox_list_get_root_path(ns->list, MAILBOX_LIST_PATH_TYPE_CONTROL,
+					&control_dir))
+		i_unreached();
 	root->maildirsize_ns = ns;
 	root->maildirsize_path =
 		p_strconcat(_root->pool, control_dir,
@@ -825,7 +842,8 @@ maildir_quota_namespace_added(struct quota *quota, struct mail_namespace *ns)
 	for (i = 0; i < count; i++) {
 		if (roots[i]->backend.name == quota_backend_maildir.name &&
 		    ((roots[i]->ns_prefix == NULL &&
-		      ns->type == NAMESPACE_PRIVATE) || roots[i]->ns == ns))
+		      ns->type == MAIL_NAMESPACE_TYPE_PRIVATE) ||
+		     roots[i]->ns == ns))
 			maildir_quota_root_namespace_added(roots[i], ns);
 	}
 }
@@ -886,14 +904,11 @@ maildir_quota_update(struct quota_root *_root,
 	} else if (root->fd == -1)
 		(void)maildirsize_recalculate(root);
 	else if (ctx->recalculate) {
-		(void)close(root->fd);
-		root->fd = -1;
+		i_close_fd(&root->fd);
 		(void)maildirsize_recalculate(root);
 	} else if (maildirsize_update(root, ctx->count_used, ctx->bytes_used) < 0) {
-		if (root->fd != -1) {
-			(void)close(root->fd);
-			root->fd = -1;
-		}
+		if (root->fd != -1)
+			i_close_fd(&root->fd);
 		maildirsize_rebuild_later(root);
 	}
 
